import { css, LitElement, property, svg } from "lit-element";
import {
  mdiCallSplit,
  mdiAbTesting,
  mdiCheck,
  mdiClose,
  mdiChevronRight,
  mdiExclamation,
  mdiTimerOutline,
  mdiTrafficLight,
  mdiRefresh,
  mdiArrowUp,
  mdiCodeJson,
  mdiCheckBoxOutline,
  mdiCheckboxBlankOutline,
  mdiAsterisk,
  mdiCircleOutline,
} from "@mdi/js";

const NODE_SIZE = 24;

const ICONS = {
  "call-split": mdiCallSplit,
  "ab-testing": mdiAbTesting,
  check: mdiCheck,
  close: mdiClose,
  "chevron-right": mdiChevronRight,
  exclamation: mdiExclamation,
  asterisk: mdiAsterisk,
};

export class HatGraphNode extends LitElement {
  @property() icon = "chevron-right";
  @property() iconPath?;

  connectedCallback() {
    super.connectedCallback();
    if (!this.hasAttribute("tabindex")) this.setAttribute("tabindex", "0");
  }

  updateNode(config) {
    this.dispatchEvent(
      new CustomEvent("update-node", {
        detail: { config },
        bubbles: true,
        composed: true,
      })
    );
  }

  deleteNode() {
    this.dispatchEvent(new CustomEvent("delete-node", { bubbles: true }));
  }

  prependNode(config) {
    this.dispatchEvent(
      new CustomEvent("prepend-node", { detail: { config }, bubbles: true })
    );
  }

  get width() {
    return NODE_SIZE + 5;
  }
  get height() {
    return NODE_SIZE + 5;
  }
  render() {
    return svg`
    <svg
      width="${this.width}"
      height="${this.height}"
      viewBox="${-this.width / 2} 0 ${this.width} ${this.height}"
    >
      <circle
        cx="0"
        cy="${this.width / 2}"
        r="${NODE_SIZE / 2}"
      />
      <g
        style="pointer-events: none"
        transform="translate(${-12} ${this.width / 2 - 12})"
      >
        ${
          this.iconPath
            ? svg`<path d="${this.iconPath}"/>`
            : ICONS[this.icon]
            ? svg`<path d="${ICONS[this.icon]}"/>`
            : ""
        }
      </g>
      </svg>
    `;
  }

  static get styles() {
    return css`
      :host {
        display: flex;
        --stroke-clr: var(--stroke-color, rgb(3, 169, 244));
        --hover-clr: var(--hover-color, rgb(255, 152, 0));
      }
      circle {
        stroke: var(--stroke-clr);
        stroke-width: 2;
        fill: white;
      }
      :host(:hover) {
        --stroke-clr: var(--hover-clr);
      }
      :host(:focus) {
        --stroke-clr: green;
        outline: none;
      }
    `;
  }
}

customElements.define("hat-graph-node", HatGraphNode);
